pub mod constants;
pub mod de;
pub mod decoder;
pub mod encoder;
pub mod reader;
pub mod ser;
pub mod signed_integer;
pub mod value;
pub mod writer;

pub mod magic;

pub use de::Deserializer;
pub use de::from_value;
pub use decoder::Decoder;
pub use encoder::Encoder;
pub use reader::BinaryReader;
pub use reader::Reader;
pub use ser::Serializer;
pub use ser::to_value;
pub use value::AnnotatedValue;
pub use value::ArcValue;
pub use value::Domain;
pub use value::IOValue;
pub use value::Map;
pub use value::NestedValue;
pub use value::NullDomain;
pub use value::PlainValue;
pub use value::RcValue;
pub use value::Set;
pub use value::UnwrappedIOValue;
pub use value::Value;
pub use writer::Writer;

pub use value::FALSE;
pub use value::TRUE;
pub use value::EMPTY_SEQ;

pub fn invert_map<A, B>(m: &Map<A, B>) -> Map<B, A>
    where A: Clone, B: Clone + Ord
{
    use std::iter::FromIterator;
    Map::from_iter(m.iter().map(|(a, b)| (b.clone(), a.clone())))
}
